@@ -1537,8 +1537,9 @@ static void GetCertificateAndKey(server_rec *s,pool *p,SSLConfigRec *pConfig)
 #if SSLEAY_VERSION_NUMBER < 0x0800
     pConfig->pSSLCtx=SSL_CTX_new();
 #else
-    pConfig->pSSLCtx=SSL_CTX_new(pConfig->bNoV2 ? SSLv3_server_method()
-				 : SSLv23_server_method());
+    pConfig->pSSLCtx=SSL_CTX_new(SSLv23_server_method());
+    if(pConfig->bNoV2)
+	SSL_CTX_set_options(pConfig->pSSLCtx,SSL_OP_NO_SSLv2);
 # if RENEG
 //    s_nVerifyClient=VERIFY_OPTIONAL_NO_CA;
     SSL_CTX_set_verify(pConfig->pSSLCtx,0,ApacheSSLVerifyCallback);
@@ -1961,6 +1962,8 @@ static int SSLSetupConnection(conn_rec *conn)
 	return TRUE;
 	}
 
+    fb->bRevoked=fb->bCRLExpired=fb->bNoCRL=FALSE;
+
     ap_conn_timeout("SSL accept timeout",conn);
 
     ReadRandomFiles(srvr,(RandomFile *)pConfig->aRandomFiles->elts,
